﻿using System;
using System.Windows.Forms;
using Tobii.Eyetracking.Sdk;

namespace Discovery.Calib
{
    public partial class CalibrationResultsForm : BaseForm
    {
        private readonly CalibrationForm _calibrationForm;
        private static TimeSpan DWELL_TIME_CALIBRATION_RESULTS_FORM = new TimeSpan(0, 0, 0, 2);


        public CalibrationResultsForm(CalibrationForm calibrationForm)
        {
            InitializeComponent();
            _calibrationForm = calibrationForm;
            _exitApplicationButton.Visible = false;
            _homeButton.Visible = false;

            KeyPreview = true;
            KeyDown += CalibrationResultsForm_KeyDown;
        }

        private void CalibrationResultsForm_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control && e.KeyCode == Keys.Add)
                DWELL_TIME_CALIBRATION_RESULTS_FORM = IncreaseDwellTime(DWELL_TIME_CALIBRATION_RESULTS_FORM);
            if (e.Control && e.KeyCode == Keys.Subtract)
                DWELL_TIME_CALIBRATION_RESULTS_FORM = DecreaseDwellTime(DWELL_TIME_CALIBRATION_RESULTS_FORM);

            UpdateDwellTimeLabel(DWELL_TIME_CALIBRATION_RESULTS_FORM);
        }

        internal override void GazeDataReceived(object sender, GazeDataEventArgs e)
        {
            base.GazeDataReceived(sender, e);

            var gd = e.GazeDataItem;

            if (GazeDataValid(gd))
            {
                if (IsUserGazingAt(_okButton))
                    DoButtonInteraction(_okButton, DWELL_TIME_CALIBRATION_RESULTS_FORM);
            }
            else
            {
                FirstTimeUserLooksAtControl = true;
                _okButton.BackgroundImage = null;
            }
        }

        public void SetPlotData(Calibration calibration)
        {
            _leftPlot.Initialize(calibration.Plot);
            _rightPlot.Initialize(calibration.Plot);
        }

        private void _okButton_Click(object sender, System.EventArgs e)
        {
            Close();
        }

        private void CalibrationResultsForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (_calibrationForm.ConnectedTracker != null)
            {
                _calibrationForm.ConnectedTracker.GazeDataReceived -= GazeDataReceived;
                _calibrationForm.ConnectedTracker.GazeDataReceived += _calibrationForm.GazeDataReceived;
            }
            _calibrationForm.Show();
        }

        private void CalibrationResultsForm_Load(object sender, EventArgs e)
        {
            UpdateDwellTimeLabel(DWELL_TIME_CALIBRATION_RESULTS_FORM);
        }
    }
}
