﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using Discovery.Properties;
using Tobii.Eyetracking.Sdk;
using System.Drawing;

namespace Discovery.Controls
{
    /*
     * This class is no longer used, but was used in Discovery v1.0.
     * 
     * The Controls Application has some commonly used Windows Controls that users can control with
     * their eyes. The main function is GazeDataReceived that overrides the base class' implementation
     * and checks whether the user stares at one of the controls or not. If the user stares at a control until the
     * dwell time has passed, an action is taken on the control.
     * 
     * Haakon Raudsandmoen and Boerge Roedsjoe, Trondheim, 16. February 2012
     */
    public partial class ControlsFormOld : BaseForm
    {
        private readonly HomeForm _homeForm;
        private readonly List<Button> _buttons;
		internal List<Control> _checkboxes;
        private Button _lastLookedAtButton;

        private static TimeSpan DWELL_TIME_CONTROLS_FORM = new TimeSpan(0, 0, 0, 1, 500);
        private static TimeSpan DWELL_TIME_TRACKBAR = new TimeSpan(0, 0, 0, 0, 500);

        private int _color1index;
        private int _color2index;
        private int _color3index;
        private int _color4index;
        private int _color5index;
        private int _color6index;

        internal IEyetracker ConnectedTracker { get; set; }
        
        // Constructor that initalizes the layout
        public ControlsFormOld(HomeForm homeForm)
        {
            InitializeComponent();
            _homeForm = homeForm;
            ConnectedTracker = _homeForm.ConnectedTracker;
            SetTimerInterval(500);

            UpdateSliderLabel();

            _lastLookedAtButton = new Button();
            _buttons = new List<Button>
                          {_homeButton, 
                           _colorButton1, 
                           _colorButton2, 
                           _colorButton3, 
                           _colorButton4, 
                           _colorButton5, 
                           _colorButton6};

            _checkboxes = new List<Control> { 
                _checkBox1, 
                _checkBox2, 
                _checkBox3, 
                _checkBox4, 
                _checkBox5, 
                _checkBox6,
                _checkBox7, 
                _checkBox8, 
                _checkBox9, 
                _checkBox10, 
                _checkBox11, 
                _checkBox12, 
                _checkBox13, 
                _checkBox14, 
                _checkBox15, 
                _checkBox16};

            KeyPreview = true;
            KeyDown += ControlsForm_KeyDown;
        }

        private void ControlsForm_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control && e.KeyCode == Keys.Add)
                DWELL_TIME_CONTROLS_FORM = IncreaseDwellTime(DWELL_TIME_CONTROLS_FORM);
            if (e.Control && e.KeyCode == Keys.Subtract)
                DWELL_TIME_CONTROLS_FORM = DecreaseDwellTime(DWELL_TIME_CONTROLS_FORM);
            if (e.Control && e.KeyCode == Keys.S)
                Log.Write("Controls Application: Saved dwell time: " + DWELL_TIME_CONTROLS_FORM.TotalMilliseconds + " ms");


            UpdateDwellTimeLabel(DWELL_TIME_CONTROLS_FORM);
        }

        // This method override's the base class' implementation, and uses the base class' functions
        // to check if the user is staring at any controls in The Controls Application. If the user
        // stares at a button, a call to DoButtonInteraction(control) is done.
        internal override void GazeDataReceived(object sender, GazeDataEventArgs e)
        {
            base.GazeDataReceived(sender, e);

            var gd = e.GazeDataItem;

            if (GazeDataValid(gd))
            {
                if (IsUserGazingAt(_homeButton))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _homeButton);
                    DoButtonInteraction(_homeButton, DWELL_TIME_CONTROLS_FORM);
                }

                else if (IsUserGazingAt(_colorButton1))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _colorButton1);
                    DoButtonInteractionForColorButtons(_colorButton1, DWELL_TIME_CONTROLS_FORM);

                    if (_lastLookedAtButton != _colorButton1)
                        _colorButton1.BackgroundImage = Resources.eye_hover;
                    _lastLookedAtButton = _colorButton1;
                }

                else if (IsUserGazingAt(_colorButton2))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _colorButton2);
                    DoButtonInteractionForColorButtons(_colorButton2, DWELL_TIME_CONTROLS_FORM);
                    
                    if (_lastLookedAtButton != _colorButton2)
                        _colorButton2.BackgroundImage = Resources.eye_hover;    
                    _lastLookedAtButton = _colorButton2;
                }

                else if (IsUserGazingAt(_colorButton3))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _colorButton3);
                    DoButtonInteractionForColorButtons(_colorButton3, DWELL_TIME_CONTROLS_FORM);

                    if (_lastLookedAtButton != _colorButton3)
                        _colorButton3.BackgroundImage = Resources.eye_hover;
                    _lastLookedAtButton = _colorButton3;
                }

                else if (IsUserGazingAt(_colorButton4))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _colorButton4);
                    DoButtonInteractionForColorButtons(_colorButton4, DWELL_TIME_CONTROLS_FORM);

                    if (_lastLookedAtButton != _colorButton4)
                        _colorButton4.BackgroundImage = Resources.eye_hover;
                    _lastLookedAtButton = _colorButton4;
                }
                
                else if (IsUserGazingAt(_colorButton5))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _colorButton5);
                    DoButtonInteractionForColorButtons(_colorButton5, DWELL_TIME_CONTROLS_FORM);

                    if (_lastLookedAtButton != _colorButton5)
                        _colorButton5.BackgroundImage = Resources.eye_hover;
                    _lastLookedAtButton = _colorButton5;
                }
                
                else if (IsUserGazingAt(_colorButton6))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _colorButton6);
                    DoButtonInteractionForColorButtons(_colorButton6, DWELL_TIME_CONTROLS_FORM);

                    if (_lastLookedAtButton != _colorButton6)
                        _colorButton6.BackgroundImage = Resources.eye_hover;
                    _lastLookedAtButton = _colorButton6;
                }
                else if (IsUserGazingAt(_trackBar1))
                {
                    if (IsUserGazingAtTrackBarValueLessThan(_trackBar1.Value) && UserHasLookedLongEnough(DWELL_TIME_TRACKBAR))
                        _trackBar1.Value -= 1;

                    else if (IsUserGazingAtTrackBarValueHigherThan(_trackBar1.Value) && UserHasLookedLongEnough(DWELL_TIME_TRACKBAR))
                        _trackBar1.Value += 1;

                    UpdateSliderLabel();
                }

                else if (IsUserGazingAt(_trackBar2))
                {
                    if (IsUserGazingAtTrackBarValueLessThan(_trackBar2.Value) && UserHasLookedLongEnough(DWELL_TIME_TRACKBAR))
                        _trackBar2.Value -= 1;

                    else if (IsUserGazingAtTrackBarValueHigherThan(_trackBar2.Value) && UserHasLookedLongEnough(DWELL_TIME_TRACKBAR))
                        _trackBar2.Value += 1;
                }
                else if (IsUserGazingAt(_linkLabel1))
                {
                    _linkLabel1.LinkBehavior = LinkBehavior.AlwaysUnderline;
                    toolTip1.Show("http://www.yr.no", _linkLabel1);

                    if (UserHasLookedLongEnough())
                    {
                        InvokeOnClick(_linkLabel1, new MouseEventArgs(MouseButtons.Left, 0, 0, 0, 0));
                        _linkLabel1.LinkBehavior = LinkBehavior.NeverUnderline;
                        toolTip1.Hide(_linkLabel1);
                    }
                }

                else if (IsUserGazingAt(_linkLabel2))
                {
                    _linkLabel2.LinkBehavior = LinkBehavior.AlwaysUnderline;
                    toolTip1.Show("http://www.ntnu.no", _linkLabel2);
                    if (UserHasLookedLongEnough())
                    {
                        InvokeOnClick(_linkLabel2, new MouseEventArgs(MouseButtons.Left, 0, 0, 0, 0));
                        _linkLabel2.LinkBehavior = LinkBehavior.NeverUnderline;
                        toolTip1.Hide(_linkLabel2);
                    }
                }

                else if (IsUserGazingAt(_linkLabel3))
                {
                    _linkLabel3.LinkBehavior = LinkBehavior.AlwaysUnderline;
                    toolTip1.Show("http://www.google.no", _linkLabel3);
                    if (UserHasLookedLongEnough())
                    {
                        InvokeOnClick(_linkLabel3, new MouseEventArgs(MouseButtons.Left, 0, 0, 0, 0));
                        _linkLabel3.LinkBehavior = LinkBehavior.NeverUnderline;
                        toolTip1.Hide(_linkLabel3);
                    }
                }

                else if (IsUserGazingAt(_linkLabel4))
                {
                    _linkLabel4.LinkBehavior = LinkBehavior.AlwaysUnderline;
                    toolTip1.Show("http://www.storm.no", _linkLabel4);
                    if (UserHasLookedLongEnough())
                    {
                        InvokeOnClick(_linkLabel4, new MouseEventArgs(MouseButtons.Left, 0, 0, 0, 0));
                        _linkLabel4.LinkBehavior = LinkBehavior.NeverUnderline;
                        toolTip1.Hide(_linkLabel4);
                    }
                }
                    
                else if (IsUserGazingAt(_checkBox1))
                {
                    _checkBox1.Font = AddBoldHover(_checkBox1.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox1.Checked = !_checkBox1.Checked;              
                }

                else if (IsUserGazingAt(_checkBox2))
                {
                    _checkBox2.Font = AddBoldHover(_checkBox2.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox2.Checked = !_checkBox2.Checked;

                }
                
                else if (IsUserGazingAt(_checkBox3))
                {
                    _checkBox3.Font = AddBoldHover(_checkBox3.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox3.Checked = !_checkBox3.Checked;
                }

                else if (IsUserGazingAt(_checkBox4))
                {
                    _checkBox4.Font = AddBoldHover(_checkBox4.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox4.Checked = !_checkBox4.Checked;
                }

                else if (IsUserGazingAt(_checkBox5))
                {
                    _checkBox5.Font = AddBoldHover(_checkBox5.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox5.Checked = !_checkBox5.Checked;
                }

                else if (IsUserGazingAt(_checkBox6))
                {
                    _checkBox6.Font = AddBoldHover(_checkBox6.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox6.Checked = !_checkBox6.Checked;
                }

                else if (IsUserGazingAt(_checkBox7))
                {
                    _checkBox7.Font = AddBoldHover(_checkBox7.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox7.Checked = !_checkBox7.Checked;
                }

                else if (IsUserGazingAt(_checkBox8))
                {
                    _checkBox8.Font = AddBoldHover(_checkBox8.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox8.Checked = !_checkBox8.Checked;
                }

                else if (IsUserGazingAt(_checkBox9))
                {
                    _checkBox9.Font = AddBoldHover(_checkBox9.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox9.Checked = !_checkBox9.Checked;
                }

                else if (IsUserGazingAt(_checkBox10))
                {
                    _checkBox10.Font = AddBoldHover(_checkBox10.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox10.Checked = !_checkBox10.Checked;
                }

                else if (IsUserGazingAt(_checkBox11))
                {
                    _checkBox11.Font = AddBoldHover(_checkBox11.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox11.Checked = !_checkBox11.Checked;
                }

                else if (IsUserGazingAt(_checkBox12))
                {
                    _checkBox12.Font = AddBoldHover(_checkBox12.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox12.Checked = !_checkBox12.Checked;
                }

                else if (IsUserGazingAt(_checkBox13))
                {
                    _checkBox13.Font = AddBoldHover(_checkBox13.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox13.Checked = !_checkBox13.Checked;
                }

                else if (IsUserGazingAt(_checkBox14))
                {
                    _checkBox14.Font = AddBoldHover(_checkBox14.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox14.Checked = !_checkBox14.Checked;
                }

                else if (IsUserGazingAt(_checkBox15))
                {
                    _checkBox15.Font = AddBoldHover(_checkBox15.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox15.Checked = !_checkBox15.Checked;
                }

                else if (IsUserGazingAt(_checkBox16))
                {
                    _checkBox16.Font = AddBoldHover(_checkBox16.Font, _checkboxes);
                    if (UserHasLookedLongEnough())
                        _checkBox16.Checked = !_checkBox16.Checked;
                }

                else
                {
                    FirstTimeUserLooksAtControl = true;
                    
                    RemoveHoverEffectFromButtons(_buttons);
                    RemoveBoldHover(_checkBox1.Font, _checkboxes);
                    
                    _lastLookedAtButton = null;

                    toolTip1.Hide(_linkLabel1);
                    toolTip1.Hide(_linkLabel2);
                    toolTip1.Hide(_linkLabel3);
                    toolTip1.Hide(_linkLabel4);
                    
                    _linkLabel1.LinkBehavior = LinkBehavior.NeverUnderline;
                    _linkLabel2.LinkBehavior = LinkBehavior.NeverUnderline;
                    _linkLabel3.LinkBehavior = LinkBehavior.NeverUnderline;
                    _linkLabel4.LinkBehavior = LinkBehavior.NeverUnderline;
                }
            }
        }

        // A helper method to determine if the user is gazing at trackbar-value higher than the
        // value passed as argument
        private bool IsUserGazingAtTrackBarValueHigherThan(int value)
        {
            var stepWidth = (_trackBar1.Width - 20) / (_trackBar1.Maximum - _trackBar1.Minimum);
            return (GazeDot.Location.X) + GazeDot.Width / 2 > _trackBar1.Parent.PointToScreen(_trackBar1.Location).X + (value * stepWidth) + stepWidth / 2;
        }

        // A helper method to determine if the user is gazing at trackbar-value less than the
        // value passed as argument
        private bool IsUserGazingAtTrackBarValueLessThan(int value)
        {
            var stepWidth = (_trackBar1.Width-20) / (_trackBar1.Maximum - _trackBar1.Minimum);
            return (GazeDot.Location.X) + GazeDot.Width / 2 < _trackBar1.Parent.PointToScreen(_trackBar1.Location).X + (value * stepWidth) - stepWidth / 2;
        }

        // If the trackbar-value is changed we update the correct label with blue colour
        private void _trackBar_ValueChanged(object sender, EventArgs e)
        {
            UpdateSliderLabel();
        }

        private void UpdateSliderLabel()
        {
            var text = "";
            switch (_trackBar1.Value)
            {
                case 0:
                    text = "15 - 20";
                    break;
                case 1:
                    text = "21 - 25";
                    break;
                case 2:
                    text = "26 - 30";
                    break;
                case 3:
                    text = "31 - 35";
                    break;
                case 4:
                    text = "36 - 40";
                    break;
                case 5:
                    text = "41 - 45";
                    break;
                case 6:
                    text = "46 - 50";
                    break;
                case 7:
                    text = "51 - 55";
                    break;
                case 8:
                    text = "56 - 60";
                    break;
                case 9:
                    text = "61 - 65";
                    break;
                case 10:
                    text = "66 - 70";
                    break;

            }
            _sliderValueLabel.Text = text;
        }

        // Close the application if home-button is clicked
        private void _homeButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        // Determines the action of what to do when one of the colourbuttons are clicked
        private void _colorButton_Click(object sender, EventArgs e)
        {
            var button = (Button) sender;
            if (button == _colorButton1)
            {
                if (_color1index >= _colors.Count)
                    _color1index = 0;
                _colorButton1.BackColor = _colors[_color1index++];
            }
            if (button == _colorButton2)
            {
                if (_color2index >= _colors.Count)
                    _color2index = 0;
                _colorButton2.BackColor = _colors[_color2index++];
            }
            if (button == _colorButton3)
            {
                if (_color3index >= _colors.Count)
                    _color3index = 0;
                _colorButton3.BackColor = _colors[_color3index++];
            }
            if (button == _colorButton4)
            {
                if (_color4index >= _colors.Count)
                    _color4index = 0;
                _colorButton4.BackColor = _colors[_color4index++];
            }
            if (button == _colorButton5)
            {
                if (_color5index >= _colors.Count)
                    _color5index = 0;
                _colorButton5.BackColor = _colors[_color5index++];
            }
            if (button == _colorButton6)
            {
                if (_color6index >= _colors.Count)
                    _color6index = 0;
                _colorButton6.BackColor = _colors[_color6index++];
            }
        }

        // If a link is clicked, we open a popup with a congratulations text
        //private void _linkLabel_Click(object sender, EventArgs e)
        //{
        //    var linkLabel = (LinkLabel) sender;

        //    if (linkLabel == _linkLabel1)
        //    {
        //        var popUpForm = new PopUpForm(this, "yr.no",
        //                      "Congratulations! \n You succsessfully clicked the link. \n \n Click OK to continue with the rest of the tasks.");
        //        if (_homeForm.ConnectedTracker != null)
        //        {
        //            _homeForm.ConnectedTracker.GazeDataReceived += popUpForm.GazeDataReceived;
        //            _homeForm.ConnectedTracker.GazeDataReceived -= GazeDataReceived;
        //        }
        //        popUpForm.Show();
        //    }
        //    if (linkLabel == _linkLabel2)
        //    {
        //        var popUpForm = new PopUpForm(this, "ntnu.no",
        //                      "Congratulations! \n You succsessfully clicked the link. \n \n Click OK to continue with the rest of the tasks.");
        //        if (_homeForm.ConnectedTracker != null)
        //        {
        //            _homeForm.ConnectedTracker.GazeDataReceived += popUpForm.GazeDataReceived;
        //            _homeForm.ConnectedTracker.GazeDataReceived -= GazeDataReceived;
        //        }
        //        popUpForm.Show();
        //    }
        //    if (linkLabel == _linkLabel3)
        //    {
        //        var popUpForm = new PopUpForm(this, "google.no",
        //                      "Congratulations! \n You succsessfully clicked the link. \n \n Click OK to continue with the rest of the tasks.");
        //        if (_homeForm.ConnectedTracker != null)
        //        {
        //            _homeForm.ConnectedTracker.GazeDataReceived += popUpForm.GazeDataReceived;
        //            _homeForm.ConnectedTracker.GazeDataReceived -= GazeDataReceived;
        //        }
        //        popUpForm.Show();
        //    }
        //    if (linkLabel == _linkLabel4)
        //    {
        //        var popUpForm = new PopUpForm(this, "storm.no",
        //                      "Congratulations! \n You succsessfully clicked the link. \n \n Click OK to continue with the rest of the tasks.");
        //        if (_homeForm.ConnectedTracker != null)
        //        {
        //            _homeForm.ConnectedTracker.GazeDataReceived += popUpForm.GazeDataReceived;
        //            _homeForm.ConnectedTracker.GazeDataReceived -= GazeDataReceived;
        //        }
        //        popUpForm.Show();
        //    }
        //}

        // Method for bold hover effect
        static internal Font AddBoldHover(Font font, List<Control> controls)
        {
            RemoveBoldHover(font, controls);
            font = new Font(font.Name, font.Size,
            FontStyle.Bold, font.Unit,
            font.GdiCharSet, font.GdiVerticalFont);

            return font;
        }
        static internal void RemoveBoldHover(Font font, List<Control> controls)
        {
            foreach (var c in controls)
            {
                c.Font = new Font(font.Name, font.Size,
                        FontStyle.Regular, font.Unit,
                        font.GdiCharSet, font.GdiVerticalFont);
            }
        }

        // Shuts down this form, stops recieving gaze data, shows the main menu and starts
        // recieving gaze data for the main menu
        private void ButtonsForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (_homeForm.ConnectedTracker != null)
            {
                _homeForm.ConnectedTracker.GazeDataReceived -= GazeDataReceived;
                _homeForm.ConnectedTracker.GazeDataReceived += _homeForm.GazeDataReceived;
            }
            _homeForm.Show();
        }

        private void ControlsForm_Load(object sender, EventArgs e)
        {
            UpdateDwellTimeLabel(DWELL_TIME_CONTROLS_FORM);
        }
    }
}
