﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using Discovery.Properties;
using Tobii.Eyetracking.Sdk;

namespace Discovery.PictureGallery
{
    public partial class PictureGalleryForm : BaseForm
    {
        private readonly List<Image> _pictures;
        private readonly List<Image> _picturesZoom1;
        private readonly List<Image> _picturesZoom2;
        private readonly List<Image> _picturesZoom3;
        private readonly List<string> _pictureTitles;
        private int _picturesIndex;
        private List<Button> _buttons; 
        
        private HomeForm _homeForm;
        internal IEyetracker ConnectedTracker { get; set; }

        private static TimeSpan DWELL_TIME_PICTURE_GALLERY = new TimeSpan(0, 0, 0, 0, 750);
        private static TimeSpan DWELL_TIME_PICTURE_GALLERY_BUTTONS = new TimeSpan(0, 0, 1, 500);

        public PictureGalleryForm(HomeForm homeForm)
        {
            InitializeComponent();
            _homeForm = homeForm;
            ConnectedTracker = _homeForm.ConnectedTracker;

            _exitApplicationButton.Visible = false;
            DoubleBuffered = true;

            _buttons = new List<Button>
                           {
                               _homeButton, 
                               _galleryPreviousButton, 
                               _galleryForwardButton
                           };

            _pictures = new List<Image>
                            {
                                new Bitmap(Resources.Chrysanthemum),
                                new Bitmap(Resources.Desert),
                                new Bitmap(Resources.Hydrangeas),
                                new Bitmap(Resources.Koala),
                                new Bitmap(Resources.Jellyfish),
                                new Bitmap(Resources.Lighthouse),
                                new Bitmap(Resources.Penguins),
                                new Bitmap(Resources.Tulips)
                            };
            _picturesZoom1 = new List<Image>
                            {
                                new Bitmap(Resources.Chrysanthemum_Zoom1),
                                new Bitmap(Resources.Desert_Zoom1),
                                new Bitmap(Resources.Hydrangeas_Zoom1),
                                new Bitmap(Resources.Koala_Zoom1),
                                new Bitmap(Resources.Jellyfish_Zoom1),
                                new Bitmap(Resources.Lighthouse_Zoom_1),
                                new Bitmap(Resources.Penguins_Zoom1),
                                new Bitmap(Resources.Tulips_Zoom1)
                            };
            _picturesZoom2 = new List<Image>
                            {
                                new Bitmap(Resources.Chrysanthemum_Zoom2),
                                new Bitmap(Resources.Desert_Zoom2),
                                new Bitmap(Resources.Hydrangeas_Zoom2),
                                new Bitmap(Resources.Koala_Zoom2),
                                new Bitmap(Resources.Jellyfish_Zoom2),
                                new Bitmap(Resources.Lighthouse_Zoom_2),
                                new Bitmap(Resources.Penguins_Zoom2),
                                new Bitmap(Resources.Tulips_Zoom2)
                            };
            _picturesZoom3 = new List<Image>
                            {
                                new Bitmap(Resources.Chrysanthemum_Zoom3),
                                new Bitmap(Resources.Desert_Zoom3),
                                new Bitmap(Resources.Hydrangeas_Zoom3),
                                new Bitmap(Resources.Koala_Zoom3),
                                new Bitmap(Resources.Jellyfish_Zoom3),
                                new Bitmap(Resources.Lighthouse_Zoom_3),
                                new Bitmap(Resources.Penguins_Zoom3),
                                new Bitmap(Resources.Tulips_Zoom3)
                            };
            _pictureTitles = new List<string>
                                {
                                    "Chrysantemum",
                                    "Desert",
                                    "Hydrangeas",
                                    "Koala",
                                    "Jellyfish",
                                    "Lighthouse",
                                    "Penguins",
                                    "Tulips"
                                };
            KeyPreview = true;
            KeyDown += PictureGallery_KeyDown;
        }

        private void PictureGallery_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control && e.KeyCode == Keys.Add)
            {
                Log.Write("Picture Gallery Application: Dwell time: " + DWELL_TIME_PICTURE_GALLERY.TotalMilliseconds + " ms");
                DWELL_TIME_PICTURE_GALLERY = IncreaseDwellTime(DWELL_TIME_PICTURE_GALLERY);
            }
            if (e.Control && e.KeyCode == Keys.Subtract)
            {
                Log.Write("Picture Gallery Application: Dwell time: " + DWELL_TIME_PICTURE_GALLERY.TotalMilliseconds + " ms");
                DWELL_TIME_PICTURE_GALLERY = DecreaseDwellTime(DWELL_TIME_PICTURE_GALLERY);
            }

            UpdateDwellTimeLabel(DWELL_TIME_PICTURE_GALLERY);
        }

        internal override void GazeDataReceived(object sender, GazeDataEventArgs e)
        {
            base.GazeDataReceived(sender, e);

            var gd = e.GazeDataItem;

            if (GazeDataValid(gd))
            {
                if (IsUserGazingAt(_homeButton))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _homeButton);
                    DoButtonInteraction(_homeButton, DWELL_TIME_PICTURE_GALLERY);
                }
                
                else if (IsUserGazingAt(_galleryPreviousButton))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _galleryPreviousButton);
                    DoButtonInteraction(_galleryPreviousButton, DWELL_TIME_PICTURE_GALLERY);
                }
                    
                else if (IsUserGazingAt(_galleryForwardButton))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _galleryForwardButton);
                    DoButtonInteraction(_galleryForwardButton, DWELL_TIME_PICTURE_GALLERY);
                }
                    
                else if (IsUserGazingAt(_pictureBox))
                {
                    var z = (int)((gd.LeftEyePosition3D.Z + gd.RightEyePosition3D.Z) / 2);
                    UpdatePictureBoxPicture(z);
                }
                else
                {
                    FirstTimeUserLooksAtControl = true;
                    RemoveHoverEffectFromButtons(_buttons);
                }
            }
        }

        private void _homeButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void _galleryPreviousButton_Click(object sender, EventArgs e)
        {
            _picturesIndex--;
            if (_picturesIndex < 0)
            {
                _picturesIndex = _pictures.Count - 1;
                _pictureBox.Image = _pictures[_picturesIndex];
            }
            else
                _pictureBox.Image = _pictures[_picturesIndex];

            _pictureTitle.Text = _pictureTitles[_picturesIndex];
        }

        private void _galleryForwardButton_Click(object sender, EventArgs e)
        {
            _picturesIndex++;
            if (_picturesIndex == _pictures.Count)
                _picturesIndex = 0;
            _pictureBox.Image = _pictures[_picturesIndex];

            _pictureTitle.Text = _pictureTitles[_picturesIndex];

        }

        private void PictureGalleryForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (_homeForm.ConnectedTracker != null)
            {
                _homeForm.ConnectedTracker.GazeDataReceived -= GazeDataReceived;
                _homeForm.ConnectedTracker.GazeDataReceived += _homeForm.GazeDataReceived;
            }
            _homeForm.Show();
        }

        private void UpdatePictureBoxPicture(int z)
        {
            if (z < 500)
                _pictureBox.Image = _picturesZoom3[_picturesIndex];

            else if (z > 500 && z < 550)
                _pictureBox.Image = _picturesZoom2[_picturesIndex];

            else if (z > 550 && z < 600)
                _pictureBox.Image = _picturesZoom1[_picturesIndex];

            else if (z > 600)
                _pictureBox.Image = _pictures[_picturesIndex];
        }

        private void PictureGalleryForm_Load(object sender, EventArgs e)
        {
            UpdateDwellTimeLabel(DWELL_TIME_PICTURE_GALLERY);
        }
    }
}