﻿using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Tobii.Eyetracking.Sdk;

namespace Discovery.TextReading
{
    public partial class TextReadingForm : BaseForm
    {
        #region Scrolling
        [return: MarshalAs(UnmanagedType.Bool)]
        [DllImport("user32.dll", SetLastError = true)]
        static extern bool PostMessage(IntPtr hWnd, uint msg, IntPtr wParam, IntPtr lParam);

        private const int WM_SCROLL = 276; // Horizontal scroll
        private const int WM_VSCROLL = 277; // Vertical scroll
        private const int SB_LINEUP = 0; // Scrolls one line up
        private const int SB_LINELEFT = 0;// Scrolls one cell left
        private const int SB_LINEDOWN = 1; // Scrolls one line down
        private const int SB_LINERIGHT = 1;// Scrolls one cell right
        private const int SB_PAGEUP = 2; // Scrolls one page up
        private const int SB_PAGELEFT = 2;// Scrolls one page left
        private const int SB_PAGEDOWN = 3; // Scrolls one page down
        private const int SB_PAGERIGTH = 3; // Scrolls one page right
        private const int SB_PAGETOP = 6; // Scrolls to the upper left
        private const int SB_LEFT = 6; // Scrolls to the left
        private const int SB_PAGEBOTTOM = 7; // Scrolls to the upper right
        private const int SB_RIGHT = 7; // Scrolls to the right
        private const int SB_ENDSCROLL = 8; // Ends scroll

        #endregion

        private readonly List<Button> _buttons;
        private static TimeSpan DWELL_TIME_TEXT_READING_BUTTONS = new TimeSpan(0, 0, 0, 1, 500);
        private HomeForm _homeForm;

        public TextReadingForm(HomeForm homeForm)
        {
            InitializeComponent();
            _homeForm = homeForm;

            _buttons = new List<Button>{_homeButton, _richTextBoxBottomButton, _richTextBoxTopButton};

            KeyPreview = true;
            KeyDown += TextReadingForm_KeyDown;
        }

        private void TextReadingForm_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control && e.KeyCode == Keys.Add)
            {
                Log.Write("Text Reading Application: Dwell time buttons: " + DWELL_TIME_TEXT_READING_BUTTONS.TotalMilliseconds);
                DWELL_TIME_TEXT_READING_BUTTONS = IncreaseDwellTime(DWELL_TIME_TEXT_READING_BUTTONS);
            }
            if (e.Control && e.KeyCode == Keys.Subtract)
            {
                Log.Write("Text Reading Application: Dwell time buttons: " + DWELL_TIME_TEXT_READING_BUTTONS.TotalMilliseconds);
                DWELL_TIME_TEXT_READING_BUTTONS = DecreaseDwellTime(DWELL_TIME_TEXT_READING_BUTTONS);
            }

            UpdateDwellTimeLabel(DWELL_TIME_TEXT_READING_BUTTONS);
        }

        internal override void GazeDataReceived(object sender, GazeDataEventArgs e)
        {
            base.GazeDataReceived(sender, e);
            
            var gd = e.GazeDataItem;

            if (GazeDataValid(gd))
            {
                if (IsUserGazingAt(_homeButton))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _homeButton);
                    DoButtonInteraction(_homeButton, DWELL_TIME_TEXT_READING_BUTTONS);
                }
                
                else if (IsUserGazingAt(_richTextBoxTopButton))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _richTextBoxTopButton);
                    DoButtonInteraction(_richTextBoxTopButton, DWELL_TIME_TEXT_READING_BUTTONS);
                }
                
                else if (IsUserGazingAt(_richTextBoxBottomButton))
                {
                    RemoveHoverEffectFromButtonsOtherThanMe(_buttons, _richTextBoxBottomButton);
                    DoButtonInteraction(_richTextBoxBottomButton, DWELL_TIME_TEXT_READING_BUTTONS);
                }
                
                else if (UserWantsToScroll(_richTextBox, "down"))
                    ScrollControl(_richTextBox, WM_VSCROLL, SB_LINEDOWN, 1);

                else if (UserWantsToScroll(_richTextBox, "up"))
                    ScrollControl(_richTextBox, WM_VSCROLL, SB_LINEUP, 1);

                else
                {
                    FirstTimeUserLooksAtControl = true;
                    RemoveHoverEffectFromButtons(_buttons);
                }
            }
        }

        private static void ScrollControl(Control control, uint verticalOrHorizontal, int upDownRightOrLeft, int i)
        {
            for (var j = 0; j < i; j++)
                PostMessage(control.Handle, verticalOrHorizontal, (IntPtr)upDownRightOrLeft, IntPtr.Zero);
        }

        private void _homeButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void _richTextBoxTopButton_Click(object sender, EventArgs e)
        {
            _richTextBox.SelectionStart = 0;
            _richTextBox.ScrollToCaret();
        }

        private void _richTextBoxBottomButton_Click(object sender, EventArgs e)
        {
            _richTextBox.SelectionStart = _richTextBox.Text.Length;
            _richTextBox.ScrollToCaret();
        }

        private void TextReadingForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (_homeForm.ConnectedTracker != null)
            {
                _homeForm.ConnectedTracker.GazeDataReceived -= GazeDataReceived;
                _homeForm.ConnectedTracker.GazeDataReceived += _homeForm.GazeDataReceived;
            }
            _homeForm.Show();
        }

        private void TextReadingForm_Load(object sender, EventArgs e)
        {
            UpdateDwellTimeLabel(DWELL_TIME_TEXT_READING_BUTTONS);
        }
    }
}
