﻿using System;
using System.Media;
using System.Windows.Forms;
using Discovery.Properties;
using Tobii.Eyetracking.Sdk;
using System.Drawing;

namespace Discovery.UFO
{
    public partial class UFOForm : BaseForm
    {
        private HomeForm _homeForm;

        private bool _gameStarted;
        private bool _first_time = true;
        private bool _finishedCountdown = true;
        private bool _newGame = true;
        private bool _soundEnabled = true;

        private int _aliens;
        private int _level = 1;
        private static TimeSpan DWELL_TIME_UFO_BUTTONS = new TimeSpan(0, 0, 0, 1, 500);

        private SoundPlayer explosion = new SoundPlayer(Resources.explode);
        private SoundPlayer pickUp = new SoundPlayer(Resources.sound72);

        public UFOForm(HomeForm homeForm)
        {
            InitializeComponent();
            _homeForm = homeForm;
            ufo_label.Text = "Welcome to UFO challange!" + Environment.NewLine +
                "Several of your beloved alien friends are drifting helplessly through space." + Environment.NewLine +
                "Pick up all 10 aliens, while avoiding the suns, to reveal your final destination." + Environment.NewLine +
                "Use your eyes to maneuver the UFO. If you look directly at it, it will stop."+ Environment.NewLine +
                Environment.NewLine +
                "Look at Start Game to begin";
            KeyPreview = true;
            KeyDown += UFO_KeyDown;
        }

        private void UFO_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control && e.KeyCode == Keys.Add)
            {
                Log.Write("UFO Application: Dwell time: " + DWELL_TIME_UFO_BUTTONS.TotalMilliseconds + " ms");
                DWELL_TIME_UFO_BUTTONS = IncreaseDwellTime(DWELL_TIME_UFO_BUTTONS);
            }
            if (e.Control && e.KeyCode == Keys.Subtract)
            {
                Log.Write("UFO Application: Dwell time: " + DWELL_TIME_UFO_BUTTONS.TotalMilliseconds + " ms");
                DWELL_TIME_UFO_BUTTONS = DecreaseDwellTime(DWELL_TIME_UFO_BUTTONS);
            }
                
            UpdateDwellTimeLabel(DWELL_TIME_UFO_BUTTONS);
        }

        internal override void GazeDataReceived(object sender, GazeDataEventArgs e)
        {
            base.GazeDataReceived(sender, e);

            var gd = e.GazeDataItem;

            if (GazeDataValid(gd))
            {
                if (IsUserGazingAt(_homeButton))
                    DoButtonInteraction(_homeButton, DWELL_TIME_UFO_BUTTONS);

                else if (IsUserGazingAt(_startGameButton))
                    DoButtonInteraction(_startGameButton, DWELL_TIME_UFO_BUTTONS);

                else if (IsUserGazingAt(_soundButton))
                    DoButtonInteraction(_soundButton, DWELL_TIME_UFO_BUTTONS);
                
                else
                {
                    FirstTimeUserLooksAtControl = true;
                    _homeButton.BackgroundImage = null;
                    _startGameButton.BackgroundImage = null;
                    _soundButton.BackgroundImage = null;
                }
            }

            //If it`s a new game; show countdown
            if (_finishedCountdown == false)
            {
                if (_newGame)
                    _newGame = false;
                RemoveUfoGraphics();
                ufo_label.Visible = false;

                //Ready-set-go code
                var countDownForm = new CountDownForm();
                _homeForm.ConnectedTracker.GazeDataReceived -= GazeDataReceived;
                var dialogResult = countDownForm.ShowDialog();
                _homeForm.ConnectedTracker.GazeDataReceived += GazeDataReceived;
                
                if (dialogResult != DialogResult.OK)
                {
                    _homeForm.ConnectedTracker.GazeDataReceived += GazeDataReceived;
                    return;
                }

                _finishedCountdown = true;
                _gameStarted = true;

            }
            //If the the start button is pressed
            if (_gameStarted)
            {
                _startGameButton.Visible = false;

                // Used to set the initial values
                if (_first_time)
                {
                    _aliens = 0;
                    ufo_label.Visible = false;
                    HideAlienIcons();
                    PrepareLevel(_level);
                    ShowLevelIcons(_level);
                    _first_time = false;
                }

                #region Collision

                if (CollisionDetection(home_pic, ufo_pic) && _aliens >= 10)
                {
                    RemoveUfoGraphics();
                    _gameStarted = false;
                    _first_time = true;
                    _level++;
                    ShowLevelIcons(_level);
                    if (_level == 5)
                    {
                        ChangeLabel(ufo_label, 16, "You are amazing! All aliens are now safely home." + Environment.NewLine
                            + "Your job is done! Look at the Home button, to return to the main menu", ContentAlignment.MiddleCenter, -1, -1, true);                        
                        _startGameButton.Text = "Close";
                    }
                    else
                    {
                        ChangeLabel(ufo_label, 16, "Pretty good!" + Environment.NewLine + "All the aliens in this galaxy are now safely home." + Environment.NewLine + Environment.NewLine
                            + "Look at the Proceed button to continue to the next level", ContentAlignment.MiddleCenter, -1, -1, true);

                        _startGameButton.Text = "Proceed";
                    }
                    _startGameButton.Visible = true;
                }
                if (SunCollision(_level, ufo_pic))
                {
                    RemoveUfoGraphics();
                    if (_soundEnabled)
                    {
                        explosion.Play();
                    }
                    _first_time = true;
                    ChangeLabel(ufo_label, 16, "Kaboom! You are dead!" + Environment.NewLine + Environment.NewLine + "Look at Restart to try again.", ContentAlignment.MiddleCenter, -1, -1, true);
                    _gameStarted = false;
                    _level = 1;
                    HideLevelIcons();
                    _startGameButton.Text = "Restart";
                    _startGameButton.Visible = true;
                }
                //UFO collides with an alien
                if (CollisionDetection(alien_pic, ufo_pic) && _aliens < 10)
                {
                    //Play sound if sound is enabeled
                    if (_soundEnabled)
                        pickUp.Play();

                    _aliens++;
                    ShowAlienIcons(_aliens);

                    //After collecting 10 aliens, show the home planet.
                    if (_aliens < 10)
                        PlaceAliens();
                    else
                    {
                        alien_pic.Visible = false;
                        home_pic.Visible = true;
                    }
                }

                #endregion

                // If not staring at the UFO, and eyes in range, move the UFO
                if (!IsUserGazingAt(ufo_pic) && GazeDataValid(gd) && IsUserGazingAt(_splitContainer.Panel2))
                {
                    //X - coordinates
                    if ((ufo_pic.Parent.PointToScreen(ufo_pic.Location).X + (ufo_pic.Width / 2)) < _eyeGazeXLowPassed)
                        ufo_pic.Location = new Point(ufo_pic.Location.X + 4, ufo_pic.Location.Y);
                    else if ((ufo_pic.Parent.PointToScreen(ufo_pic.Location).X + (ufo_pic.Width / 2)) > _eyeGazeXLowPassed)
                        ufo_pic.Location = new Point(ufo_pic.Location.X - 4, ufo_pic.Location.Y);
                    //Y - coordinates
                    if ((ufo_pic.Parent.PointToScreen(ufo_pic.Location).Y + (ufo_pic.Height / 2)) > _eyeGazeYLowPassed)
                        ufo_pic.Location = new Point(ufo_pic.Location.X, ufo_pic.Location.Y - 4);
                    else if ((ufo_pic.Parent.PointToScreen(ufo_pic.Location).Y + (ufo_pic.Height / 2)) < _eyeGazeYLowPassed)
                        ufo_pic.Location = new Point(ufo_pic.Location.X, ufo_pic.Location.Y + 4);
                }
            }
        }

        // Return a random number between the min and max parameters.
        private static int RandomNumber(int min, int max)
        {
            var random = new Random();
            return random.Next(min, max);
        }
        // Changes a labels properties.
        internal void ChangeLabel(Label label, float fontSize, String text, ContentAlignment textAlign, int locX, int locY, Boolean visability)
        {
            if (fontSize != 0) label.Font = ChangeFontSize(label.Font, fontSize);
            if (text != "") label.Text = text;
            if (locX != -1 && locY != -1) label.Location = new Point(locX, locY);
            
            label.Visible = visability;
            label.TextAlign = textAlign;
        }

        // Method for changing font size
        static internal Font ChangeFontSize(Font font, float fontSize)
        {
            if (font != null)
            {
                var currentSize = font.Size;
                if (currentSize != fontSize)
                {
                    font = new Font(font.Name, fontSize,
                        font.Style, font.Unit,
                        font.GdiCharSet, font.GdiVerticalFont);
                }
            }
            return font;
        }
        // Method for changing pictureBox size
        private void ChangePBSize(PictureBox pictureBox, int width, int height)
        {
            // Get the size in a temp variable.
            Size tempSize = pictureBox.Size;

            // Set the Width.
            tempSize.Width = width;
            tempSize.Height = height;

            // Set the size back.
            pictureBox.Size = tempSize;
        }

        // Removes UFO graphics elements
        private void RemoveUfoGraphics()
        {
            ufo_pic.Visible = false;
            alien_pic.Visible = false;
            home_pic.Visible = false;
            sun_pic.Visible = false;
            sun_pic2.Visible = false;
            sun_pic3.Visible = false;
            sun_pic_big1.Visible = false;
            sun_pic_big2.Visible = false;
            info_label1.Visible = false;
            info_label2.Visible = false;
            info_label3.Visible = false;
            info_label4.Visible = false;

        }
        // Levels
        private void PrepareLevel(int level)
        {
            ufo_pic.Location = new Point(_splitContainer.Panel2.Width / 2, _splitContainer.Panel2.Height / 2);
            ufo_pic.Visible = true;            
            alien_pic.Visible = true;
            PlaceAliens();
            home_pic.Location = new Point(100, 500);

            switch (level)
            {
                case 1:
                    sun_pic.Location = new Point(400, 500);
                    sun_pic.Visible = true;
                    sun_pic2.Location = new Point(700, 200);
                    sun_pic2.Visible = true;
                    break;
                case 2:
                    sun_pic.Location = new Point(400, 500);
                    sun_pic.Visible = true;
                    sun_pic2.Location = new Point(700, 200);
                    sun_pic2.Visible = true;
                    sun_pic3.Location = new Point(250, 350);
                    sun_pic3.Visible = true;
                    break;
                case 3:
                    ufo_pic.Location = new Point(0, 0);
                    sun_pic_big1.Location = new Point(500, 300);
                    sun_pic_big1.Visible = true;
                    break;
                case 4:
                    ufo_pic.Location = new Point(0, 0);
                    sun_pic_big1.Location = new Point(600, 300);
                    sun_pic_big1.Visible = true;
                    sun_pic_big2.Location = new Point(200, 300);
                    sun_pic_big2.Visible = true;
                    break;
            }
        }

        // Collision detection
        internal bool CollisionDetection(Control A, Control B)
        {
            return (A.Bounds.Contains(B.Location.X, B.Location.Y)) ||
                   (A.Bounds.Contains(B.Location.X + B.Width, B.Location.Y + B.Height)) ||
                   (A.Bounds.Contains(B.Location.X + B.Width, B.Location.Y)) ||
                   (A.Bounds.Contains(B.Location.X, B.Location.Y + B.Height)) ||
                   (B.Bounds.Contains(A.Location.X, A.Location.Y)) ||
                   (B.Bounds.Contains(A.Location.X + A.Width, A.Location.Y + A.Height)) ||
                   (B.Bounds.Contains(A.Location.X + A.Width, A.Location.Y)) ||
                   (B.Bounds.Contains(A.Location.X, A.Location.Y + A.Height));
        }

        //Dependent on the level, checks if a pictureBox collides with a Sun
        private bool SunCollision(int level, PictureBox pB)
        {
            var crash = false;
            if (level == 1)
                crash = CollisionDetection(pB, sun_pic) || CollisionDetection(pB, sun_pic2);
            else if (level == 2)
                crash = CollisionDetection(pB, sun_pic) || CollisionDetection(pB, sun_pic2) || CollisionDetection(pB, sun_pic3);
            else if (level == 3)
                crash = CollisionDetection(pB, sun_pic_big1);
            else if (level == 4)
                crash = CollisionDetection(pB, sun_pic_big1) || CollisionDetection(pB, sun_pic_big2);
            return crash;
        }

        //Decide where to place the random aliens
        private void PlaceAliens()
        {
            alien_pic.Location = new Point(RandomNumber(20, 770), RandomNumber(20, 450));
            while (SunCollision(_level, alien_pic))
            {
                alien_pic.Location = new Point(RandomNumber(20, 770), RandomNumber(20, 450));
            }
        }

        //Hiding all alien-icons.
        private void HideAlienIcons()
        {
            alien1.Visible = false;
            alien2.Visible = false;
            alien3.Visible = false;
            alien4.Visible = false;
            alien5.Visible = false;
            alien6.Visible = false;
            alien7.Visible = false;
            alien8.Visible = false;
            alien9.Visible = false;
            alien10.Visible = false;
        }

        //Hiding all level-icons.
        private void HideLevelIcons()
        {
            level1.Visible = false;
            level2.Visible = false;
            level3.Visible = false;
            level4.Visible = false;
            level5.Visible = false;
            level6.Visible = false;
            level7.Visible = false;
            level8.Visible = false;
            level9.Visible = false;
            level10.Visible = false;
        }

        //Showing aliens based on the alien counter.
        private void ShowAlienIcons(int aliens)
        {
            switch (aliens)
            {
                case 1:
                    alien1.Visible = true;
                    break;
                case 2:
                    alien2.Visible = true;
                    break;
                case 3:
                    alien3.Visible = true;
                    break;
                case 4:
                    alien4.Visible = true;
                    break;
                case 5:
                    alien5.Visible = true;
                    break;
                case 6:
                    alien6.Visible = true;
                    break;
                case 7:
                    alien7.Visible = true;
                    break;
                case 8:
                    alien8.Visible = true;
                    break;
                case 9:
                    alien9.Visible = true;
                    break;
                case 10:
                    alien10.Visible = true;
                    break;
            }
        }
        //Showing level-icons based on the level counter.
        private void ShowLevelIcons(int level)
        {
            switch (level)
            {
                case 1:
                    level1.Visible = true;
                    break;
                case 2:
                    level2.Visible = true;
                    break;
                case 3:
                    level3.Visible = true;
                    break;
                case 4:
                    level4.Visible = true;
                    break;
                case 5:
                    level5.Visible = true;
                    break;
                case 6:
                    level6.Visible = true;
                    break;
                case 7:
                    level7.Visible = true;
                    break;
                case 8:
                    level8.Visible = true;
                    break;
                case 9:
                    level9.Visible = true;
                    break;
                case 10:
                    level10.Visible = true;
                    break;
            }
        }
        private void startGameButton_Click(object sender, EventArgs e)
        {
            if (_startGameButton.Text == "Close")
            {
                _finishedCountdown = true;
                Close();
            }
            else
            {
                _finishedCountdown = false;
                _newGame = true;
            }

        }
        private void soundButton_Click(object sender, EventArgs e)
        {
            if (_soundEnabled)
            {
                _soundButton.Image = Resources.audio_m;
                _soundEnabled = false;
            }
            else
            {
                _soundButton.Image = Resources.audio;
                _soundEnabled = true;
            }
        }
        private void _homeButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void UFOForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (_homeForm.ConnectedTracker != null)
            {
                _homeForm.ConnectedTracker.GazeDataReceived -= GazeDataReceived;
                _homeForm.ConnectedTracker.GazeDataReceived += _homeForm.GazeDataReceived;
            }
            _homeForm.Show();
        }

        private void UFOForm_Load(object sender, EventArgs e)
        {
            if (GazeDot.Visible)
                GazeDot.Visible = false;
            UpdateDwellTimeLabel(DWELL_TIME_UFO_BUTTONS);
        }
    }
}
